package com.example;

import com.example.config.AppConfig;
import com.example.config.NacosConfig;
import com.example.exception.BusinessException;
import com.example.exception.GlobalExceptionHandler;
import com.example.exception.ResourceNotFoundException;
import com.example.exception.ValidationException;
import com.example.route.RoleRoute;
import com.example.route.UserRoute;
import io.vertx.core.Vertx;
import io.vertx.core.VertxOptions;
import org.apache.camel.CamelContext;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.platform.http.vertx.VertxPlatformHttpServer;
import org.apache.camel.component.platform.http.vertx.VertxPlatformHttpServerConfiguration;
import org.apache.camel.component.properties.PropertiesComponent;
import org.apache.camel.component.vertx.VertxComponent;
import org.apache.camel.impl.DefaultCamelContext;
import org.apache.camel.model.rest.RestBindingMode;
import org.apache.camel.support.DefaultHeaderFilterStrategy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeoutException;

/**
 * 创建使用camelContext
 */
public class Application1 {
    private static final Logger logger = LoggerFactory.getLogger(Application.class);
    
    public static void main(String[] args) {
        try {
            // 获取本机IP地址
            String hostName = InetAddress.getLocalHost().getHostName();
            String hostAddress = InetAddress.getLocalHost().getHostAddress();
            
            // 配置Vert.x选项
            VertxOptions options = new VertxOptions()
                    .setWorkerPoolSize(20)
                    .setEventLoopPoolSize(2);
            
            // 创建Vert.x实例
            Vertx vertx = Vertx.vertx(options);
            
            // 服务端口和主机
            String serverHost = AppConfig.getProperty("server.host", "0.0.0.0");
            int serverPort = AppConfig.getIntProperty("server.port", 8080);
            
            logger.info("服务器将绑定到: {}:{}", serverHost, serverPort);
            
            // 创建Camel上下文
            DefaultCamelContext camelContext = new DefaultCamelContext();
            
            // 设置属性
            Properties properties = new Properties();
            properties.setProperty("server.host", serverHost);
            properties.setProperty("server.port", String.valueOf(serverPort));
            
            PropertiesComponent propertiesComponent = new PropertiesComponent();
            propertiesComponent.setInitialProperties(properties);
            camelContext.setPropertiesComponent(propertiesComponent);
            
            // 配置Vertx组件
            VertxComponent vertxComponent = new VertxComponent();
            vertxComponent.setVertx(vertx);
            camelContext.addComponent("vertx", vertxComponent);
            
            // 创建并注册platform-http组件
            org.apache.camel.component.platform.http.PlatformHttpComponent platformHttpComponent = 
                new org.apache.camel.component.platform.http.PlatformHttpComponent();
            platformHttpComponent.setHeaderFilterStrategy(new DefaultHeaderFilterStrategy());
            camelContext.addComponent("platform-http", platformHttpComponent);
            
            // 配置Vert.x Platform HTTP Server
            VertxPlatformHttpServerConfiguration httpConfig = new VertxPlatformHttpServerConfiguration();
            httpConfig.setBindPort(serverPort);
            httpConfig.setBindHost(serverHost);
            
            logger.info("正在创建HTTP服务器，配置: 主机={}, 端口={}", serverHost, serverPort);
            
            // 创建并配置 HTTP 服务器
            VertxPlatformHttpServer httpServer = new VertxPlatformHttpServer(httpConfig);
            httpServer.setVertx(vertx);
            
            // 将HTTP服务器注册到Camel
            camelContext.getRegistry().bind("vertx-platform-http-server", httpServer);
            
            // 创建主HTTP服务器实例
            io.vertx.core.http.HttpServer mainHttpServer = vertx.createHttpServer();
            
            // 创建和注册Vert.x Router
            io.vertx.ext.web.Router vertxRouter = io.vertx.ext.web.Router.router(vertx);
            
            // 添加 Body 处理器，确保可以处理请求体
            vertxRouter.route().handler(io.vertx.ext.web.handler.BodyHandler.create());
            
            // 添加健康检查端点
            vertxRouter.route("/health").handler(ctx -> {
                ctx.response()
                    .putHeader("Content-Type", "application/json")
                    .end("{\"status\":\"UP\",\"message\":\"服务运行正常\"}");
            });
            
            // 添加 CORS 处理器
            io.vertx.ext.web.handler.CorsHandler corsHandler = io.vertx.ext.web.handler.CorsHandler.create()
                .addOrigin("*")  // 允许所有来源
                .allowedMethod(io.vertx.core.http.HttpMethod.GET)
                .allowedMethod(io.vertx.core.http.HttpMethod.POST)
                .allowedMethod(io.vertx.core.http.HttpMethod.PUT)
                .allowedMethod(io.vertx.core.http.HttpMethod.DELETE)
                .allowedMethod(io.vertx.core.http.HttpMethod.OPTIONS)
                .allowCredentials(true)
                .allowedHeader("*");  // 允许所有头信息
            vertxRouter.route().handler(corsHandler);
            
            // 添加请求日志处理器
            vertxRouter.route("/*").handler(ctx -> {
                if (logger.isDebugEnabled()) {
                    logger.debug("收到请求: {} {}", ctx.request().method(), ctx.request().uri());
                }
                ctx.next(); // 继续处理请求
            });
            
            // 添加 API 路径处理器
            vertxRouter.route("/api/*").handler(ctx -> {
                ctx.next(); // 直接传递给下一个处理器，由 Camel 接管
            });
            
            // 添加错误处理器
            vertxRouter.errorHandler(404, ctx -> {
                ctx.response()
                    .putHeader("Content-Type", "application/json")
                    .setStatusCode(404)
                    .end("{\"error\":\"未找到请求的资源\",\"path\":\"" + ctx.request().uri() + "\"}");
            });
            
            vertxRouter.errorHandler(400, ctx -> {
                ctx.response()
                    .putHeader("Content-Type", "application/json")
                    .setStatusCode(400)
                    .end("{\"error\":\"请求参数错误\",\"path\":\"" + ctx.request().uri() + "\"}");
            });
            
            vertxRouter.errorHandler(500, ctx -> {
                ctx.response()
                    .putHeader("Content-Type", "application/json")
                    .setStatusCode(500)
                    .end("{\"error\":\"服务器内部错误\",\"message\":\"" + 
                        (ctx.failure() != null ? ctx.failure().getMessage() : "未知错误") + "\"}");
            });
            
            // 创建并注册VertxPlatformHttpRouter
            logger.info("创建VertxPlatformHttpRouter");
            try {
                org.apache.camel.component.platform.http.vertx.VertxPlatformHttpRouter platformHttpRouter = 
                    new org.apache.camel.component.platform.http.vertx.VertxPlatformHttpRouter(httpServer, vertx, vertxRouter);
                camelContext.getRegistry().bind("platform-http-router", platformHttpRouter);
                logger.info("成功将VertxPlatformHttpRouter绑定到Camel注册表");
                
                // 将Router绑定到主HTTP服务器
                mainHttpServer.requestHandler(vertxRouter).listen(serverPort, serverHost, res -> {
                    if (res.succeeded()) {
                        logger.info("主HTTP服务器成功启动，监听在 {}:{}", serverHost, serverPort);
                    } else {
                        logger.error("主HTTP服务器启动失败: {}", res.cause().getMessage());
                    }
                });
            } catch (Exception e) {
                logger.error("创建或注册VertxPlatformHttpRouter时发生错误", e);
                throw e;
            }
            
            // 添加统一的REST配置
            logger.info("开始添加REST路由配置");
            try {
                camelContext.addRoutes(new RouteBuilder() {
                    @Override
                    public void configure() throws Exception {
                        // 配置全局REST设置
                        restConfiguration()
                            .component("platform-http")
                            .bindingMode(RestBindingMode.json)
                            .enableCORS(true)
                            .dataFormatProperty("prettyPrint", "true")
                            .apiContextPath("/api-doc")
                            .apiContextRouteId("api-doc")
                            .contextPath("/api");
                            
                        logger.info("全局REST配置已完成: component=platform-http, contextPath=/api");
                        
                        // 添加一个测试端点
                        rest()
                            .id("test-route")
                            .get("test")
                                .description("测试REST端点")
                                .produces("application/json")
                                .to("direct:test");
                        
                        // 处理测试请求
                        from("direct:test")
                            .process(exchange -> {
                                Map<String, Object> response = new HashMap<>();
                                response.put("status", "OK");
                                response.put("message", "REST API工作正常");
                                response.put("timestamp", System.currentTimeMillis());
                                exchange.getMessage().setBody(response);
                            });
                    }
                });
                logger.info("REST路由添加成功");
            } catch (Exception e) {
                logger.error("添加REST路由时发生错误", e);
                throw e;
            }
            
            // 添加全局异常处理
            camelContext.addRoutes(new RouteBuilder() {
                @Override
                public void configure() throws Exception {
                    // 配置全局异常处理
                    GlobalExceptionHandler exceptionHandler = new GlobalExceptionHandler();
                    
                    // 处理各种异常类型
                    onException(ValidationException.class)
                        .handled(true)
                        .process(exceptionHandler);
                        
                    onException(ResourceNotFoundException.class)
                        .handled(true)
                        .process(exceptionHandler);
                        
                    onException(BusinessException.class)
                        .handled(true)
                        .process(exceptionHandler);
                        
                    onException(SecurityException.class)
                        .handled(true)
                        .process(exceptionHandler);
                        
                    onException(TimeoutException.class)
                        .handled(true)
                        .process(exceptionHandler);
                        
                    onException(IllegalArgumentException.class)
                        .handled(true)
                        .process(exceptionHandler);
                        
                    // 处理所有其他未捕获的异常
                    onException(Exception.class)
                        .handled(true)
                        .process(exceptionHandler);
                }
            });
            
            // 添加业务路由
            camelContext.addRoutes(new UserRoute());
            camelContext.addRoutes(new RoleRoute());
            
            // 启动Camel
            logger.info("正在启动Camel上下文...");
            try {
                camelContext.start();
                logger.info("Camel上下文启动成功");
            } catch (Exception e) {
                logger.error("Camel上下文启动失败", e);
                throw e;
            }
            
            logger.info("应用程序已成功启动，HTTP服务器监听地址: {}:{}", serverHost, serverPort);
            
            // 注册服务到Nacos
            String serviceName = AppConfig.getProperty("nacos.service.name", "camel-vertx-service");
            String registryIp = "0.0.0.0".equals(serverHost) || "localhost".equals(serverHost) ? 
                    hostAddress : serverHost;
            
            Map<String, String> metadata = new HashMap<>();
            metadata.put("application.name", "camel-vertx-demo");
            metadata.put("version", "1.0.0");
            metadata.put("hostname", hostName);
            
            boolean registered = NacosConfig.registerInstance(serviceName, registryIp, serverPort, metadata);
            if (registered) {
                logger.info("服务已成功注册到Nacos: {}:{}", registryIp, serverPort);
            } else {
                logger.warn("服务注册到Nacos失败，将继续运行但不提供服务注册");
            }
            
            // 检查服务健康状态
            boolean isHealthy = NacosConfig.isServiceHealthy(serviceName);
            if (isHealthy) {
                logger.info("服务健康状态正常");
            } else {
                logger.warn("服务可能不健康，请检查");
                
                // 添加重试逻辑，等待服务变为健康状态
                vertx.setPeriodic(1000, timerId -> {
                    boolean healthStatus = NacosConfig.isServiceHealthy(serviceName);
                    if (healthStatus) {
                        logger.info("服务健康状态已恢复正常");
                        vertx.cancelTimer(timerId);
                    }
                });
            }
            
            // 关闭钩子
            Runtime.getRuntime().addShutdownHook(new Thread(() -> {
                try {
                    logger.info("关闭应用程序...");
                    
                    // 从Nacos注销服务
                    NacosConfig.deregisterInstance(serviceName, registryIp, serverPort);
                    
                    camelContext.stop();
                    vertx.close();
                    logger.info("应用程序已关闭");
                } catch (Exception e) {
                    logger.error("关闭应用程序时发生错误", e);
                }
            }));
            
            // 阻塞主线程，直到收到终止信号
            Thread.currentThread().join();
            
        } catch (Exception e) {
            logger.error("应用程序启动失败", e);
            System.exit(1);
        }
    }
}