package com.example.config;

import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import io.vertx.core.Vertx;
import io.vertx.pgclient.PgConnectOptions;
import io.vertx.pgclient.PgPool;
import io.vertx.sqlclient.PoolOptions;
import org.apache.ibatis.mapping.Environment;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.apache.ibatis.transaction.jdbc.JdbcTransactionFactory;

import javax.sql.DataSource;
import java.io.IOException;
import java.io.InputStream;

public class DatabaseConfig {
    private static SqlSessionFactory sqlSessionFactory;
    private static PgPool pgPool;
    
    public static SqlSessionFactory getSqlSessionFactory() {
        if (sqlSessionFactory == null) {
            synchronized (DatabaseConfig.class) {
                if (sqlSessionFactory == null) {
                    try {
                        DataSource dataSource = createDataSource();
                        
                        // 加载MyBatis配置
                        try (InputStream inputStream = DatabaseConfig.class.getClassLoader().getResourceAsStream("mybatis-config.xml")) {
                            // 从输入流创建SqlSessionFactory
                            sqlSessionFactory = new SqlSessionFactoryBuilder().build(inputStream);
                            
                            // 更新环境设置
                            Configuration configuration = sqlSessionFactory.getConfiguration();
                            Environment environment = new Environment("development", new JdbcTransactionFactory(), dataSource);
                            configuration.setEnvironment(environment);
                        }
                    } catch (IOException e) {
                        throw new RuntimeException("初始化MyBatis失败", e);
                    }
                }
            }
        }
        return sqlSessionFactory;
    }
    
    public static PgPool getPgPool(Vertx vertx) {
        if (pgPool == null) {
            synchronized (DatabaseConfig.class) {
                if (pgPool == null) {
                    // 解析 JDBC URL 以提取连接参数
                    String jdbcUrl = AppConfig.getProperty("database.url", "jdbc:postgresql://localhost:5432/camel_demo");
                    String[] urlParts = jdbcUrl.replace("jdbc:postgresql://", "").split("/");
                    String hostAndPort = urlParts[0];
                    String database = urlParts[1];
                    
                    String host = "localhost";
                    int port = 5432;
                    
                    if (hostAndPort.contains(":")) {
                        String[] hostPortParts = hostAndPort.split(":");
                        host = hostPortParts[0];
                        port = Integer.parseInt(hostPortParts[1]);
                    } else {
                        host = hostAndPort;
                    }
                    
                    PgConnectOptions connectOptions = new PgConnectOptions()
                            .setPort(port)
                            .setHost(host)
                            .setDatabase(database)
                            .setUser(AppConfig.getProperty("database.username", "postgres"))
                            .setPassword(AppConfig.getProperty("database.password", "postgres"));

                    PoolOptions poolOptions = new PoolOptions()
                            .setMaxSize(10);

                    pgPool = PgPool.pool(vertx, connectOptions, poolOptions);
                }
            }
        }
        return pgPool;
    }
    
    private static DataSource createDataSource() {
        System.out.println("正在创建数据库连接池...");
        HikariConfig config = new HikariConfig();
        
        // 使用新的配置属性名
        String jdbcUrl = AppConfig.getProperty("database.url", "jdbc:postgresql://localhost:5432/camel_demo");
        String username = AppConfig.getProperty("database.username", "postgres");
        String password = AppConfig.getProperty("database.password", "postgres");
        String driver = AppConfig.getProperty("database.driver", "org.postgresql.Driver");
        
        System.out.println("数据库配置: jdbcUrl=" + jdbcUrl);
        System.out.println("数据库配置: username=" + username);
        System.out.println("数据库配置: driver=" + driver);
        
        config.setJdbcUrl(jdbcUrl);
        config.setUsername(username);
        config.setPassword(password);
        config.setDriverClassName(driver);
        
        // 池配置
        config.setMaximumPoolSize(10);
        config.setMinimumIdle(2);
        config.setIdleTimeout(30000);
        config.setConnectionTestQuery("SELECT 1");
        config.setAutoCommit(true);
        
        System.out.println("数据库连接池创建完成");
        return new HikariDataSource(config);
    }
} 