package com.example.config;

import com.alibaba.nacos.api.NacosFactory;
import com.alibaba.nacos.api.exception.NacosException;
import com.alibaba.nacos.api.naming.NamingService;
import com.alibaba.nacos.api.naming.pojo.Instance;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Nacos服务注册配置类
 */
public class NacosConfig {
    private static final Logger logger = LoggerFactory.getLogger(NacosConfig.class);
    private static NamingService namingService;
    
    /**
     * 获取Nacos命名服务
     */
    public static NamingService getNamingService() {
        // 从配置文件中读取Nacos配置
        String serverAddr = AppConfig.getProperty("nacos.server-addr", "localhost:8848");
        String namespace = AppConfig.getProperty("nacos.namespace", "public");
        int beatThreadCount = AppConfig.getIntProperty("nacos.naming.client.beat.thread.count", 1);
        int beatInterval = AppConfig.getIntProperty("nacos.naming.client.beat.interval", 5000);
        int maxRetryTimes = AppConfig.getIntProperty("nacos.naming.client.max.retry.times", 3);
        int timeout = AppConfig.getIntProperty("nacos.naming.client.timeout", 15000);
        
        return getNamingService(serverAddr, namespace, beatThreadCount, beatInterval, maxRetryTimes, timeout);
    }
    
    /**
     * 获取Nacos命名服务（带参数）
     */
    public static NamingService getNamingService(String serverAddr, String namespace, 
                                                 int beatThreadCount, int beatInterval, 
                                                 int maxRetryTimes, int timeout) {
        if (namingService == null) {
            synchronized (NacosConfig.class) {
                if (namingService == null) {
                    try {
                        Properties properties = new Properties();
                        properties.setProperty("serverAddr", serverAddr);
                        properties.setProperty("namespace", namespace);
                        
                        // Nacos 2.4.3中推荐使用的配置名称
                        properties.setProperty("naming.client.beat.thread.count", String.valueOf(beatThreadCount));
                        properties.setProperty("naming.client.beat.interval", String.valueOf(beatInterval));
                        properties.setProperty("naming.client.max.retry.times", String.valueOf(maxRetryTimes));
                        properties.setProperty("naming.client.timeout", String.valueOf(timeout));
                        
                        // 在2.4.3中也可以同时兼容老的配置名称
                        properties.setProperty("namingClientBeatThreadCount", String.valueOf(beatThreadCount));
                        properties.setProperty("namingClientBeatInterval", String.valueOf(beatInterval));
                        properties.setProperty("namingClientMaxRetryTimes", String.valueOf(maxRetryTimes));
                        properties.setProperty("namingClientTimeout", String.valueOf(timeout));
                        
                        logger.debug("正在创建Nacos命名服务，配置参数: {}", properties);
                        namingService = NacosFactory.createNamingService(properties);
                        logger.info("Nacos命名服务已创建，服务地址: {}", serverAddr);
                    } catch (NacosException e) {
                        logger.error("创建Nacos命名服务失败", e);
                        throw new RuntimeException("创建Nacos命名服务失败", e);
                    }
                }
            }
        }
        return namingService;
    }
    
    /**
     * 注册服务实例
     * 
     * @param serviceName 服务名称
     * @param ip IP地址
     * @param port 端口
     * @param metadata 元数据
     * @return 是否注册成功
     */
    public static boolean registerInstance(String serviceName, String ip, int port, Map<String, String> metadata) {
        try {
            logger.debug("开始构建Nacos服务实例: {} ({}:{})", serviceName, ip, port);
            
            Instance instance = new Instance();
            instance.setIp(ip);
            instance.setPort(port);
            instance.setWeight(1.0);
            instance.setEnabled(true);
            instance.setHealthy(true);
            
            // 设置集群名称 - 从配置文件读取
            String clusterName = AppConfig.getProperty("nacos.cluster-name", "DEFAULT");
            instance.setClusterName(clusterName);
            logger.debug("设置服务实例集群名称: {}", clusterName);
            
            // 设置元数据
            if (metadata == null) {
                metadata = new HashMap<>();
            }
            
            // 添加必要的元数据
            metadata.put("preserved.register.source", "CAMEL_VERTX");
            
            // 在2.4.3中，心跳参数通过properties设置，这里不再需要在metadata中设置
            // 但为兼容性考虑，仍然保留此参数
            int heartBeatInterval = AppConfig.getIntProperty("nacos.naming.client.beat.interval", 5000);
            int heartBeatTimeout = Math.max(heartBeatInterval * 2, 
                    AppConfig.getIntProperty("nacos.naming.client.timeout", 15000));
            int ipDeleteTimeout = Math.max(heartBeatTimeout * 2, 30000); // IP删除超时应该大于心跳超时
            
            metadata.put("preserved.heart.beat.interval", String.valueOf(heartBeatInterval));
            metadata.put("preserved.heart.beat.timeout", String.valueOf(heartBeatTimeout));
            metadata.put("preserved.ip.delete.timeout", String.valueOf(ipDeleteTimeout));
            
            logger.debug("服务心跳配置 - 间隔: {}ms, 超时: {}ms, IP删除超时: {}ms", 
                    heartBeatInterval, heartBeatTimeout, ipDeleteTimeout);
            
            instance.setMetadata(metadata);
            
            // 获取服务分组 - 从配置文件读取
            String serviceGroup = AppConfig.getProperty("nacos.service.group", "DEFAULT_GROUP");
            
            logger.debug("开始注册服务实例到Nacos: {}@@{} ({}:{}), 元数据: {}", 
                    serviceGroup, serviceName, ip, port, metadata);
            
            // 注册实例
            getNamingService().registerInstance(serviceName, serviceGroup, instance);
            logger.info("服务实例已成功注册到Nacos: {}@@{} ({}:{})", serviceGroup, serviceName, ip, port);
            return true;
        } catch (Exception e) {
            logger.error("注册服务实例到Nacos失败: {}", serviceName, e);
            return false;
        }
    }
    
    /**
     * 注销服务实例
     * 
     * @param serviceName 服务名称
     * @param ip IP地址
     * @param port 端口
     * @return 是否注销成功
     */
    public static boolean deregisterInstance(String serviceName, String ip, int port) {
        try {
            String serviceGroup = AppConfig.getProperty("nacos.service.group", "DEFAULT_GROUP");
            logger.debug("开始从Nacos注销服务实例: {}@@{} ({}:{})",
                    serviceGroup, serviceName, ip, port);
            
            // 在2.4.3版本中，推荐使用此方式注销实例
            getNamingService().deregisterInstance(serviceName, serviceGroup, ip, port);
            logger.info("服务实例已从Nacos注销: {}@@{} ({}:{})", serviceGroup, serviceName, ip, port);
            return true;
        } catch (Exception e) {
            logger.error("从Nacos注销服务实例失败: {}", serviceName, e);
            return false;
        }
    }
    
    /**
     * 获取服务健康状态
     * 
     * @param serviceName 服务名称
     * @return 是否健康
     */
    public static boolean isServiceHealthy(String serviceName) {
        try {
            String serviceGroup = AppConfig.getProperty("nacos.service.group", "DEFAULT_GROUP");
            logger.debug("检查服务健康状态: {}@@{}", serviceGroup, serviceName);
            
            // 直接查询实例列表来判断服务是否存在和健康
            try {
                boolean isHealthy = !getNamingService().selectInstances(serviceName, serviceGroup, true).isEmpty();
                logger.debug("服务 {}@@{} 健康状态: {}", serviceGroup, serviceName, isHealthy ? "健康" : "不健康");
                return isHealthy;
            } catch (com.alibaba.nacos.api.exception.NacosException e) {
                // 如果是因为客户端未连接或正在启动，不记录详细错误堆栈
                if (e.getMessage() != null && e.getMessage().contains("Client not connected")) {
                    logger.debug("Nacos客户端尚未完全连接: {}", e.getMessage());
                } else {
                    logger.error("查询服务健康状态失败: {}", serviceName, e);
                }
                return false;
            }
        } catch (Exception e) {
            logger.error("查询服务健康状态失败: {}", serviceName, e);
            return false;
        }
    }
} 