package com.example.exception;

import io.vertx.core.json.JsonObject;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeoutException;

/**
 * 全局异常处理器
 * 用于统一处理系统中产生的各种异常
 */
public class GlobalExceptionHandler implements Processor {
    private static final Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @Override
    public void process(Exchange exchange) throws Exception {
        Exception exception = exchange.getProperty(Exchange.EXCEPTION_CAUGHT, Exception.class);
        
        if (exception instanceof ValidationException) {
            handleValidationException(exchange, (ValidationException) exception);
        } else if (exception instanceof ResourceNotFoundException) {
            handleResourceNotFoundException(exchange, (ResourceNotFoundException) exception);
        } else if (exception instanceof BusinessException) {
            handleBusinessException(exchange, (BusinessException) exception);
        } else if (exception instanceof SecurityException) {
            handleSecurityException(exchange, (SecurityException) exception);
        } else if (exception instanceof TimeoutException) {
            handleTimeoutException(exchange, (TimeoutException) exception);
        } else if (exception instanceof IllegalArgumentException) {
            handleIllegalArgumentException(exchange, (IllegalArgumentException) exception);
        } else {
            handleUnknownException(exchange, exception);
        }
    }
    
    /**
     * 处理验证异常
     */
    private void handleValidationException(Exchange exchange, ValidationException exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", 400);
        response.put("message", exception.getMessage());
        response.put("errors", exception.getErrors());
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 400);
        exchange.getMessage().setBody(response);
        logger.warn("验证异常: {}", exception.getMessage());
    }
    
    /**
     * 处理资源未找到异常
     */
    private void handleResourceNotFoundException(Exchange exchange, ResourceNotFoundException exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", 404);
        response.put("message", exception.getMessage());
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 404);
        exchange.getMessage().setBody(response);
        logger.warn("资源未找到: {}", exception.getMessage());
    }
    
    /**
     * 处理业务异常
     */
    private void handleBusinessException(Exchange exchange, BusinessException exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", exception.getStatusCode());
        response.put("message", exception.getMessage());
        response.put("code", exception.getErrorCode());
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, exception.getStatusCode());
        exchange.getMessage().setBody(response);
        logger.warn("业务异常 [{}]: {}", exception.getErrorCode(), exception.getMessage());
    }
    
    /**
     * 处理安全异常
     */
    private void handleSecurityException(Exchange exchange, SecurityException exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", 403);
        response.put("message", "您没有权限执行此操作");
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 403);
        exchange.getMessage().setBody(response);
        logger.warn("安全异常: {}", exception.getMessage());
    }
    
    /**
     * 处理超时异常
     */
    private void handleTimeoutException(Exchange exchange, TimeoutException exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", 408);
        response.put("message", "请求处理超时，请稍后重试");
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 408);
        exchange.getMessage().setBody(response);
        logger.error("请求超时: {}", exception.getMessage());
    }
    
    /**
     * 处理参数异常
     */
    private void handleIllegalArgumentException(Exchange exchange, IllegalArgumentException exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", 400);
        response.put("message", exception.getMessage());
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 400);
        exchange.getMessage().setBody(response);
        logger.warn("参数异常: {}", exception.getMessage());
    }
    
    /**
     * 处理未知异常
     */
    private void handleUnknownException(Exchange exchange, Exception exception) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", true);
        response.put("status", 500);
        response.put("message", "服务器内部错误，请稍后重试");
        
        if (logger.isDebugEnabled()) {
            // 在开发环境中可以返回更详细的错误信息
            response.put("exception", exception.getClass().getName());
            response.put("detail", exception.getMessage());
        }
        
        exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 500);
        exchange.getMessage().setBody(response);
        logger.error("未知异常: ", exception);
    }
} 