package com.example.route;

import com.example.config.DatabaseConfig;
import com.example.exception.ResourceNotFoundException;
import com.example.validation.RequestValidator;
import io.vertx.core.json.JsonArray;
import io.vertx.core.json.JsonObject;
import org.apache.camel.Exchange;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.model.rest.RestBindingMode;
import org.apache.camel.model.rest.RestParamType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RoleRoute extends RouteBuilder {
    private static final Logger logger = LoggerFactory.getLogger(RoleRoute.class);
    private final SqlSessionFactory sqlSessionFactory;
    
    public RoleRoute() {
        this.sqlSessionFactory = DatabaseConfig.getSqlSessionFactory();
    }
    
    @Override
    public void configure() throws Exception {
        // 删除REST配置部分，只保留路由定义
        // 现在使用全局REST配置
        
        // 使用无前缀路径，由全局配置处理前缀
        rest()
            .id("role-route")
            .tag("role")
            .description("角色管理API")
            .consumes("application/json")
            .produces("application/json")
            
            // 获取所有角色
            .get("roles")
                .description("获取所有角色")
                .outType(Map.class)
                .to("direct:getAllRoles")
                
            // 分页获取角色
            .get("roles/page")
                .description("分页获取角色")
                .param().name("page").type(RestParamType.valueOf("query")).description("页码").defaultValue("1").endParam()
                .param().name("size").type(RestParamType.valueOf("query")).description("每页大小").defaultValue("10").endParam()
                .outType(Map.class)
                .to("direct:getRolesByPage")
                
            // 根据ID获取角色
            .get("roles/{id}")
                .description("根据ID获取角色")
                .outType(Map.class)
                .to("direct:getRoleById")
                
            // 创建角色
            .post("roles")
                .description("创建新角色")
                .type(Map.class)
                .outType(Map.class)
                .to("direct:createRole")
                
            // 更新角色
            .put("roles/{id}")
                .description("更新角色")
                .type(Map.class)
                .outType(Map.class)
                .to("direct:updateRole")
                
            // 删除角色
            .delete("roles/{id}")
                .description("删除角色")
                .outType(Void.class)
                .to("direct:deleteRole");
                
        // 实现路由处理逻辑
        from("direct:getAllRoles")
            .id("getAllRolesRoute")
            .log("处理获取所有角色请求")
            .process(exchange -> {
                try (SqlSession sqlSession = sqlSessionFactory.openSession()) {
                    List<Map<String, Object>> roles = sqlSession.selectList("RoleMapper.findAll");
                    Map<String, Object> result = new HashMap<>();
                    result.put("roles", roles);
                    exchange.getMessage().setBody(result);
                } catch (Exception e) {
                    logger.error("查询所有角色失败", e);
                    exchange.setException(e);
                }
            });
            
        from("direct:getRolesByPage")
            .id("getRolesByPageRoute")
            .log("处理分页获取角色请求")
            .process(exchange -> {
                // 获取分页参数
                String pageStr = exchange.getIn().getHeader("page", "1", String.class);
                String sizeStr = exchange.getIn().getHeader("size", "10", String.class);
                
                // 验证分页参数
                RequestValidator.validate()
                    .field("page", pageStr)
                    .notEmpty("页码不能为空")
                    .custom(value -> {
                        try {
                            return Integer.parseInt(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "页码必须是正整数")
                    .field("size", sizeStr)
                    .notEmpty("每页大小不能为空")
                    .custom(value -> {
                        try {
                            int sizeVal = Integer.parseInt(value);
                            return sizeVal > 0 && sizeVal <= 100;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "每页大小必须在1-100之间")
                    .validate("分页参数验证失败");
                
                int page = Integer.parseInt(pageStr);
                int size = Integer.parseInt(sizeStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession()) {
                    Map<String, Object> params = new HashMap<>();
                    int offset = (page - 1) * size;
                    params.put("offset", offset);
                    params.put("limit", size);
                    
                    List<Map<String, Object>> roles = sqlSession.selectList("RoleMapper.findByPage", params);
                    int total = sqlSession.selectOne("RoleMapper.countAll");
                    
                    JsonObject result = new JsonObject();
                    result.put("content", new JsonArray(roles));
                    result.put("page", page);
                    result.put("size", size);
                    result.put("total", total);
                    result.put("totalPages", (int) Math.ceil((double) total / size));
                    
                    exchange.getMessage().setBody(result.getMap());
                } catch (Exception e) {
                    logger.error("分页查询角色失败", e);
                    exchange.setException(e);
                }
            });
            
        from("direct:getRoleById")
            .id("getRoleByIdRoute")
            .log("处理根据ID获取角色请求: ${header.id}")
            .process(exchange -> {
                String idStr = exchange.getIn().getHeader("id", String.class);
                
                // 验证ID参数
                RequestValidator.validate()
                    .field("id", idStr)
                    .notEmpty("角色ID不能为空")
                    .custom(value -> {
                        try {
                            return Long.parseLong(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "角色ID必须是正整数")
                    .validate("角色ID验证失败");
                
                Long id = Long.parseLong(idStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession()) {
                    Map<String, Object> role = sqlSession.selectOne("RoleMapper.findById", id);
                    
                    if (role == null) {
                        exchange.setException(new ResourceNotFoundException("角色", id));
                    } else {
                        exchange.getMessage().setBody(role);
                    }
                } catch (Exception e) {
                    logger.error("根据ID查询角色失败, id: {}", id, e);
                    exchange.setException(e);
                }
            });
            
        from("direct:createRole")
            .id("createRoleRoute")
            .log("处理创建角色请求")
            .process(exchange -> {
                Map<String, Object> body = exchange.getIn().getBody(Map.class);
                JsonObject json = new JsonObject(body);
                
                // 验证角色数据
                RequestValidator.validate()
                    .field("name", json.getString("name", ""))
                    .notEmpty("角色名称不能为空")
                    .length(2, 50, "角色名称长度必须在2-50个字符之间")
                    
                    .field("code", json.getString("code", ""))
                    .notEmpty("角色编码不能为空")
                    .length(2, 50, "角色编码长度必须在2-50个字符之间")
                    .matches("[A-Z_]+", "角色编码必须是大写字母和下划线组合")
                    
                    .validate("角色数据验证失败");
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession(true)) {
                    Map<String, Object> params = json.getMap();
                    sqlSession.insert("RoleMapper.insert", params);
                    
                    // 返回创建后的角色信息（包含ID）
                    Long id = ((Number) params.get("id")).longValue();
                    Map<String, Object> newRole = sqlSession.selectOne("RoleMapper.findById", id);
                    
                    exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 201);
                    exchange.getMessage().setBody(newRole);
                } catch (Exception e) {
                    logger.error("创建角色失败", e);
                    exchange.setException(e);
                }
            });
            
        from("direct:updateRole")
            .id("updateRoleRoute")
            .log("处理更新角色请求: ${header.id}")
            .process(exchange -> {
                String idStr = exchange.getIn().getHeader("id", String.class);
                Map<String, Object> body = exchange.getIn().getBody(Map.class);
                JsonObject json = new JsonObject(body);
                
                // 验证ID参数
                RequestValidator.validate()
                    .field("id", idStr)
                    .notEmpty("角色ID不能为空")
                    .custom(value -> {
                        try {
                            return Long.parseLong(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "角色ID必须是正整数")
                    .validate("角色ID验证失败");
                
                // 验证角色数据
                RequestValidator.validate()
                    .field("name", json.getString("name", ""))
                    .notEmpty("角色名称不能为空")
                    .length(2, 50, "角色名称长度必须在2-50个字符之间")
                    
                    .field("code", json.getString("code", ""))
                    .notEmpty("角色编码不能为空")
                    .length(2, 50, "角色编码长度必须在2-50个字符之间")
                    .matches("[A-Z_]+", "角色编码必须是大写字母和下划线组合")
                    
                    .validate("角色数据验证失败");
                
                Long id = Long.parseLong(idStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession(true)) {
                    // 检查角色是否存在
                    Map<String, Object> existingRole = sqlSession.selectOne("RoleMapper.findById", id);
                    if (existingRole == null) {
                        exchange.setException(new ResourceNotFoundException("角色", id));
                        return;
                    }
                    
                    // 更新角色信息
                    Map<String, Object> params = json.getMap();
                    params.put("id", id);
                    sqlSession.update("RoleMapper.update", params);
                    
                    // 返回更新后的角色信息
                    Map<String, Object> updatedRole = sqlSession.selectOne("RoleMapper.findById", id);
                    exchange.getMessage().setBody(updatedRole);
                } catch (Exception e) {
                    logger.error("更新角色失败, id: {}", id, e);
                    exchange.setException(e);
                }
            });
            
        from("direct:deleteRole")
            .id("deleteRoleRoute")
            .log("处理删除角色请求: ${header.id}")
            .process(exchange -> {
                String idStr = exchange.getIn().getHeader("id", String.class);
                
                // 验证ID参数
                RequestValidator.validate()
                    .field("id", idStr)
                    .notEmpty("角色ID不能为空")
                    .custom(value -> {
                        try {
                            return Long.parseLong(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "角色ID必须是正整数")
                    .validate("角色ID验证失败");
                
                Long id = Long.parseLong(idStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession(true)) {
                    // 检查角色是否存在
                    Map<String, Object> existingRole = sqlSession.selectOne("RoleMapper.findById", id);
                    if (existingRole == null) {
                        exchange.setException(new ResourceNotFoundException("角色", id));
                        return;
                    }
                    
                    // 删除角色
                    sqlSession.delete("RoleMapper.delete", id);
                    exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 204);
                    exchange.getMessage().setBody(null);
                } catch (Exception e) {
                    logger.error("删除角色失败, id: {}", id, e);
                    exchange.setException(e);
                }
            });
    }
} 