package com.example.route;

import com.example.config.DatabaseConfig;
import com.example.exception.ResourceNotFoundException;
import com.example.validation.RequestValidator;
import io.vertx.core.json.JsonArray;
import io.vertx.core.json.JsonObject;
import org.apache.camel.Exchange;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.model.rest.RestBindingMode;
import org.apache.camel.model.rest.RestParamType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class UserRoute extends RouteBuilder {
    private static final Logger logger = LoggerFactory.getLogger(UserRoute.class);
    private final SqlSessionFactory sqlSessionFactory;
    
    public UserRoute() {
        this.sqlSessionFactory = DatabaseConfig.getSqlSessionFactory();
    }
    
    @Override
    public void configure() throws Exception {
        // 删除REST配置部分，只保留路由定义
        // 现在使用全局REST配置
        
        // 使用无前缀路径，由全局配置处理前缀
        rest()
            .id("user-route")
            .tag("user")
            .description("用户管理API")
            .consumes("application/json")
            .produces("application/json")
            
            // 获取所有用户
            .get("users")
                .description("获取所有用户")
                .outType(Map.class)
                .to("direct:getAllUsers")
                
            // 分页获取用户
            .get("users/page")
                .description("分页获取用户")
                .param().name("page").type(RestParamType.valueOf("query")).description("页码").defaultValue("1").endParam()
                .param().name("size").type(RestParamType.valueOf("query")).description("每页大小").defaultValue("10").endParam()
                .outType(Map.class)
                .to("direct:getUsersByPage")
                
            // 根据ID获取用户
            .get("users/{id}")
                .description("根据ID获取用户")
                .outType(Map.class)
                .to("direct:getUserById")
                
            // 创建用户
            .post("users")
                .description("创建新用户")
                .type(Map.class)
                .outType(Map.class)
                .to("direct:createUser")
                
            // 更新用户
            .put("users/{id}")
                .description("更新用户")
                .type(Map.class)
                .outType(Map.class)
                .to("direct:updateUser")
                
            // 删除用户
            .delete("users/{id}")
                .description("删除用户")
                .outType(Void.class)
                .to("direct:deleteUser");
                
        // 实现路由处理逻辑
        from("direct:getAllUsers")
            .id("getAllUsersRoute")
            .log("处理获取所有用户请求")
            .process(exchange -> {
                try (SqlSession sqlSession = sqlSessionFactory.openSession()) {
                    List<Map<String, Object>> users = sqlSession.selectList("UserMapper.findAll");
                    Map<String, Object> result = new HashMap<>();
                    result.put("users", users);
                    exchange.getMessage().setBody(result);
                } catch (Exception e) {
                    logger.error("查询所有用户失败", e);
                    exchange.setException(e);
                }
            });
            
        from("direct:getUsersByPage")
            .id("getUsersByPageRoute")
            .log("处理分页获取用户请求")
            .process(exchange -> {
                // 获取分页参数
                String pageStr = exchange.getIn().getHeader("page", "1", String.class);
                String sizeStr = exchange.getIn().getHeader("size", "10", String.class);
                
                // 验证分页参数
                RequestValidator.validate()
                    .field("page", pageStr)
                    .notEmpty("页码不能为空")
                    .custom(value -> {
                        try {
                            return Integer.parseInt(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "页码必须是正整数")
                    .field("size", sizeStr)
                    .notEmpty("每页大小不能为空")
                    .custom(value -> {
                        try {
                            int sizeVal = Integer.parseInt(value);
                            return sizeVal > 0 && sizeVal <= 100;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "每页大小必须在1-100之间")
                    .validate("分页参数验证失败");
                
                int page = Integer.parseInt(pageStr);
                int size = Integer.parseInt(sizeStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession()) {
                    Map<String, Object> params = new HashMap<>();
                    int offset = (page - 1) * size;
                    params.put("offset", offset);
                    params.put("limit", size);
                    
                    List<Map<String, Object>> users = sqlSession.selectList("UserMapper.findByPage", params);
                    int total = sqlSession.selectOne("UserMapper.countAll");
                    
                    JsonObject result = new JsonObject();
                    result.put("content", new JsonArray(users));
                    result.put("page", page);
                    result.put("size", size);
                    result.put("total", total);
                    result.put("totalPages", (int) Math.ceil((double) total / size));
                    
                    exchange.getMessage().setBody(result.getMap());
                } catch (Exception e) {
                    logger.error("分页查询用户失败", e);
                    exchange.setException(e);
                }
            });
            
        from("direct:getUserById")
            .id("getUserByIdRoute")
            .log("处理根据ID获取用户请求: ${header.id}")
            .process(exchange -> {
                String idStr = exchange.getIn().getHeader("id", String.class);
                
                // 验证ID参数
                RequestValidator.validate()
                    .field("id", idStr)
                    .notEmpty("用户ID不能为空")
                    .custom(value -> {
                        try {
                            return Long.parseLong(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "用户ID必须是正整数")
                    .validate("用户ID验证失败");
                
                Long id = Long.parseLong(idStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession()) {
                    Map<String, Object> user = sqlSession.selectOne("UserMapper.findById", id);
                    
                    if (user == null) {
                        exchange.setException(new ResourceNotFoundException("用户", id));
                    } else {
                        exchange.getMessage().setBody(user);
                    }
                } catch (Exception e) {
                    logger.error("根据ID查询用户失败, id: {}", id, e);
                    exchange.setException(e);
                }
            });
            
        from("direct:createUser")
            .id("createUserRoute")
            .log("处理创建用户请求")
            .process(exchange -> {
                Map<String, Object> body = exchange.getIn().getBody(Map.class);
                JsonObject json = new JsonObject(body);
                
                // 验证用户数据
                RequestValidator.validate()
                    .field("username", json.getString("username", ""))
                    .notEmpty("用户名不能为空")
                    .length(3, 50, "用户名长度必须在3-50个字符之间")
                    
                    .field("email", json.getString("email", ""))
                    .notEmpty("邮箱不能为空")
                    .email("邮箱格式不正确")
                    
                    .field("phone", json.getString("phone", ""))
                    .custom(value -> value == null || value.isEmpty() || RequestValidator.isValidPhone(value), 
                            "手机号格式不正确")
                    
                    .validate("用户数据验证失败");
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession(true)) {
                    Map<String, Object> params = json.getMap();
                    sqlSession.insert("UserMapper.insert", params);
                    
                    // 返回创建后的用户信息（包含ID）
                    Long id = ((Number) params.get("id")).longValue();
                    Map<String, Object> newUser = sqlSession.selectOne("UserMapper.findById", id);
                    
                    exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 201);
                    exchange.getMessage().setBody(newUser);
                } catch (Exception e) {
                    logger.error("创建用户失败", e);
                    exchange.setException(e);
                }
            });
            
        from("direct:updateUser")
            .id("updateUserRoute")
            .log("处理更新用户请求: ${header.id}")
            .process(exchange -> {
                String idStr = exchange.getIn().getHeader("id", String.class);
                Map<String, Object> body = exchange.getIn().getBody(Map.class);
                JsonObject json = new JsonObject(body);
                
                // 验证ID参数
                RequestValidator.validate()
                    .field("id", idStr)
                    .notEmpty("用户ID不能为空")
                    .custom(value -> {
                        try {
                            return Long.parseLong(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "用户ID必须是正整数")
                    .validate("用户ID验证失败");
                
                // 验证用户数据
                RequestValidator.validate()
                    .field("username", json.getString("username", ""))
                    .notEmpty("用户名不能为空")
                    .length(3, 50, "用户名长度必须在3-50个字符之间")
                    
                    .field("email", json.getString("email", ""))
                    .notEmpty("邮箱不能为空")
                    .email("邮箱格式不正确")
                    
                    .field("phone", json.getString("phone", ""))
                    .custom(value -> value == null || value.isEmpty() || RequestValidator.isValidPhone(value), 
                            "手机号格式不正确")
                    
                    .validate("用户数据验证失败");
                
                Long id = Long.parseLong(idStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession(true)) {
                    // 检查用户是否存在
                    Map<String, Object> existingUser = sqlSession.selectOne("UserMapper.findById", id);
                    if (existingUser == null) {
                        exchange.setException(new ResourceNotFoundException("用户", id));
                        return;
                    }
                    
                    // 更新用户信息
                    Map<String, Object> params = json.getMap();
                    params.put("id", id);
                    sqlSession.update("UserMapper.update", params);
                    
                    // 返回更新后的用户信息
                    Map<String, Object> updatedUser = sqlSession.selectOne("UserMapper.findById", id);
                    exchange.getMessage().setBody(updatedUser);
                } catch (Exception e) {
                    logger.error("更新用户失败, id: {}", id, e);
                    exchange.setException(e);
                }
            });
            
        from("direct:deleteUser")
            .id("deleteUserRoute")
            .log("处理删除用户请求: ${header.id}")
            .process(exchange -> {
                String idStr = exchange.getIn().getHeader("id", String.class);
                
                // 验证ID参数
                RequestValidator.validate()
                    .field("id", idStr)
                    .notEmpty("用户ID不能为空")
                    .custom(value -> {
                        try {
                            return Long.parseLong(value) > 0;
                        } catch (NumberFormatException e) {
                            return false;
                        }
                    }, "用户ID必须是正整数")
                    .validate("用户ID验证失败");
                
                Long id = Long.parseLong(idStr);
                
                try (SqlSession sqlSession = sqlSessionFactory.openSession(true)) {
                    // 检查用户是否存在
                    Map<String, Object> existingUser = sqlSession.selectOne("UserMapper.findById", id);
                    if (existingUser == null) {
                        exchange.setException(new ResourceNotFoundException("用户", id));
                        return;
                    }
                    
                    // 删除用户
                    sqlSession.delete("UserMapper.delete", id);
                    exchange.getMessage().setHeader(Exchange.HTTP_RESPONSE_CODE, 204);
                    exchange.getMessage().setBody(null);
                } catch (Exception e) {
                    logger.error("删除用户失败, id: {}", id, e);
                    exchange.setException(e);
                }
            });
    }
} 