package com.example.validation;

import com.example.exception.ValidationException;
import io.vertx.core.json.JsonObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import java.util.regex.Pattern;

/**
 * 通用请求参数验证工具类
 */
public class RequestValidator {
    // 邮箱正则表达式
    private static final Pattern EMAIL_PATTERN = Pattern.compile("^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,6}$");
    
    // 手机号正则表达式（中国大陆）
    private static final Pattern PHONE_PATTERN = Pattern.compile("^1[3-9]\\d{9}$");
    
    // 密码复杂度正则表达式（至少8位，包含大小写字母、数字和特殊字符）
    private static final Pattern PASSWORD_STRENGTH_PATTERN = 
            Pattern.compile("^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d)(?=.*[@$!%*?&])[A-Za-z\\d@$!%*?&]{8,}$");
    
    // ID卡号正则表达式（18位）
    private static final Pattern ID_CARD_PATTERN = Pattern.compile("^\\d{17}[0-9X]$");

    /**
     * 创建一个新的验证构建器
     * @return 验证构建器
     */
    public static ValidationBuilder validate() {
        return new ValidationBuilder();
    }
    
    /**
     * 检查字符串是否是有效的电子邮件
     * @param email 电子邮件字符串
     * @return 是否有效
     */
    public static boolean isValidEmail(String email) {
        return email != null && EMAIL_PATTERN.matcher(email).matches();
    }
    
    /**
     * 检查字符串是否是有效的手机号码
     * @param phone 手机号码字符串
     * @return 是否有效
     */
    public static boolean isValidPhone(String phone) {
        return phone != null && PHONE_PATTERN.matcher(phone).matches();
    }
    
    /**
     * 检查密码是否符合复杂度要求
     * @param password 密码字符串
     * @return 是否有效
     */
    public static boolean isValidPasswordStrength(String password) {
        return password != null && PASSWORD_STRENGTH_PATTERN.matcher(password).matches();
    }
    
    /**
     * 检查字符串是否匹配指定的正则表达式
     * @param value 字符串值
     * @param pattern 正则表达式
     * @return 是否匹配
     */
    public static boolean matchesPattern(String value, String pattern) {
        return value != null && Pattern.compile(pattern).matcher(value).matches();
    }
    
    /**
     * 检查字符串是否是有效的ID卡号
     * @param idCard ID卡号字符串
     * @return 是否有效
     */
    public static boolean isValidIdCard(String idCard) {
        return idCard != null && ID_CARD_PATTERN.matcher(idCard).matches();
    }
    
    /**
     * 验证构建器，用于链式调用验证方法
     */
    public static class ValidationBuilder {
        private final Map<String, List<String>> errors = new HashMap<>();
        private String currentField;
        private String currentValue;
        
        /**
         * 设置当前要验证的字段
         * @param fieldName 字段名
         * @param value 字段值
         * @return 当前构建器实例
         */
        public ValidationBuilder field(String fieldName, String value) {
            this.currentField = fieldName;
            this.currentValue = value;
            return this;
        }
        
        /**
         * 验证字段非空
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder notEmpty(String message) {
            if (currentValue == null || currentValue.trim().isEmpty()) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 验证字段长度在指定范围内
         * @param min 最小长度
         * @param max 最大长度
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder length(int min, int max, String message) {
            if (currentValue != null && (currentValue.length() < min || currentValue.length() > max)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 验证字段是有效的电子邮件
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder email(String message) {
            if (currentValue != null && !isValidEmail(currentValue)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 验证字段是有效的手机号码
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder phone(String message) {
            if (currentValue != null && !isValidPhone(currentValue)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 验证字段符合密码复杂度要求
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder passwordStrength(String message) {
            if (currentValue != null && !isValidPasswordStrength(currentValue)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 验证字段是有效的ID卡号
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder idCard(String message) {
            if (currentValue != null && !isValidIdCard(currentValue)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 验证字段匹配指定的正则表达式
         * @param pattern 正则表达式
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder matches(String pattern, String message) {
            if (currentValue != null && !matchesPattern(currentValue, pattern)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 添加自定义验证规则
         * @param predicate 验证条件
         * @param message 错误消息
         * @return 当前构建器实例
         */
        public ValidationBuilder custom(Predicate<String> predicate, String message) {
            if (currentValue != null && !predicate.test(currentValue)) {
                addError(message);
            }
            return this;
        }
        
        /**
         * 添加错误信息
         * @param message
         */
        private void addError(String message) {
            errors.computeIfAbsent(currentField, k -> new ArrayList<>()).add(message);
        }
        
        /**
         * 如果有错误，抛出验证异常
         * @param message 验证失败的整体消息
         */
        public void validate(String message) {
            if (!errors.isEmpty()) {
                throw new ValidationException(message, errors);
            }
        }
        
        /**
         * 检查是否有错误
         * @return 是否有错误
         */
        public boolean hasErrors() {
            return !errors.isEmpty();
        }
        
        /**
         * 获取所有错误
         * @return 错误映射
         */
        public Map<String, List<String>> getErrors() {
            return errors;
        }
    }
} 